// Hippo header file.


// libio headers
#include <iostream>
#include <fstream>

// STL headers
#include <string>
#include <vector>
#include <map>
#include <functional>
#include <algorithm>

// Forward declarations
class song;
class songlist;
class playlist;
class playermap;
class interface;
class interface_text;


// A song within a list.
class song {
public:
	string filename;
	string name;
};

// A list of songs, which can be selected from using a playlist.
class songlist : public vector<song> {
public:
	playermap *pm;

	songlist(playermap *newpm);

	void select_all(playlist *pl);
	void select_match(playlist *pl, const string& key);
	void add(const string& filename);
	void read_list(const string& filename);
	void play(int index, playlist *caller);
	void stop();
};

// A list of indexes into a songlist.
class playlist : public vector<int> {
public:
	enum sort_methods {
		by_name,
		by_filename
	};

	subtractive_rng rng;
	iterator pos;
	songlist *sl;
	bool continuous;
	bool playing;

	playlist(songlist *newsl);

	void reset();
	void select(int num);
	int play();
	int play_next();
	int play_previous();
	void stop();
	void shuffle();
	void song_finished();
	void set_continuous(bool newc);
	const song& get_song(int index);
	void sort(enum sort_methods method);
};

// A class mapping file extensions to players.
class playermap : public map<string, string> {
public:
	string play_command(const string& filename);
	string song_name(string filename);
};

// The base class for all interfaces.
class interface {
public:
	songlist *sl;

	interface(songlist *newsl);
	virtual ~interface();

	virtual void create(int& argc, char**& argv);
	virtual void main();
	virtual void song_changed(int index); // index within current playlist
};

// The text-mode interface.
class interface_text : public interface {
public:
	playlist *pl;

	interface_text(songlist *newsl);
	~interface_text();

	void main();
	void song_changed(int index);
};

// The Motif interface.
class interface_motif : public interface {
 public:
  playlist *pl;

  interface_motif(songlist *newsl);
  ~interface_motif();

  void create(int& argc, char**& argv);
  void main();
  void song_changed(int index);

  void fill_list();
};

// The GTK interface.
class interface_gtk : public interface {
 public:
  playlist *pl;

  // The last song picked, to avoid endless loops with the 
  // list-click handler.
  int last_song;

  interface_gtk(songlist *newsl);
  ~interface_gtk();

  void create(int& argc, char**& argv);
  void main();
  void song_changed(int index);

  void fill_list();
};

// Extern variables.
// In main.cpp:
extern interface *iface;

// Other function prototypes.
// In main.cpp:
int main(int argc, char **argv);
// In subprocess.cpp:
void subprocess_start(const string& command, playlist *newlist);
void subprocess_stop();
void subprocess_sigchld();
void subprocess_init();
