// Hippo Motif interface.

#include "hippo.h"

extern "C" {
#include <Xm/Xm.h>
#include <Xm/Form.h>
#include <Xm/PushB.h>
#include <Xm/List.h>
#include <Xm/Text.h>
}

// The Motif interface.
interface_motif *moif;

// Xt widgets
Widget app_shell, form;
Widget prev_button, next_button, stop_button, shuffle_button, sort_button;
Widget search_text, all_button, listview;

// Xt application context
XtAppContext app;

interface_motif::interface_motif(songlist *newsl) : interface(newsl) {
  pl = new playlist(newsl);
  newsl->select_all(pl);

  moif = this;
}

interface_motif::~interface_motif() {
  delete pl;
}

// Called when the list is double-clicked.
void listpick_CB(Widget widget, XtPointer app_data, XtPointer widget_data) {

  // Get the selected position.
  int pos = ((XmListCallbackStruct *)widget_data)->item_position - 1;

  // Select and play the song.
  moif->pl->select(pos);
  moif->pl->play();
}

// Button callbacks.
void next_CB(Widget w, XtPointer ad, XtPointer wd) { 
  moif->pl->play_next(); 
}
void prev_CB(Widget w, XtPointer ad, XtPointer wd) { 
  moif->pl->play_previous(); 
}
void stop_CB(Widget w, XtPointer ad, XtPointer wd) { 
  moif->pl->stop(); 
}
void shuffle_CB(Widget w, XtPointer ad, XtPointer wd) { 
  moif->pl->shuffle(); moif->fill_list(); 
}
void sort_CB(Widget w, XtPointer ad, XtPointer wd) { 
  moif->pl->sort(playlist::by_name); moif->fill_list(); 
}
void all_CB(Widget w, XtPointer ad, XtPointer wd) { 
  moif->sl->select_all(moif->pl); moif->fill_list();
}
void search_CB(Widget w, XtPointer ad, XtPointer wd) { 
  char *key = XmTextGetString(w);
  moif->sl->select_match(moif->pl, key);
  XtFree(key);
  moif->fill_list();
}

// Timeout handler that updates the song.
void changesong_CB(XtPointer data, XtIntervalId *id) {
  XmListSelectPos(listview, (int)data + 1, False);
  XmListSetPos(listview, (int)data + 1);
  
  // Flush pending display events so that the list is redrawn.
  XFlush(XtDisplay(listview));
}

// Fill the listview with the items in the playlist.
void interface_motif::fill_list() {
  unsigned int i;

  // Clear the listview.
  XmListDeleteAllItems(listview);

  // Create an array of XmStrings for the names.
  unsigned int s = pl->size();
  XmString names[s];
  for(i=0; i<s; i++) names[i] = XmStringCreateSimple((char *)pl->get_song(i).name.c_str());

  // Add the names to the list.
  XmListAddItemsUnselected(listview, names, s, 0);

  // Free the array.
  for(i=0; i<s; i++) XmStringFree(names[i]);
}

void interface_motif::create(int& argc, char**& argv) {
  XmString xs;

  // Initialise Motif application.
  app_shell = XtAppInitialize(&app, "XHippo", (XrmOptionDescList)NULL,
			      0, &argc, argv,
			      (String *)NULL, (ArgList)NULL, 0);

  // Form to pack the interface into.
  form = XtVaCreateWidget("mainform", xmFormWidgetClass, app_shell,
			  XmNheight, 300,
			  NULL);

  // Prev button.
  xs = XmStringCreateSimple("Prev");
  prev_button = XtVaCreateWidget("prevbutton", xmPushButtonWidgetClass, form,
				 XmNlabelString, xs,
				 XmNleftAttachment, XmATTACH_FORM,
				 XmNtopAttachment, XmATTACH_FORM,
				 NULL);
  XmStringFree(xs);
  XtAddCallback(prev_button, XmNactivateCallback,
		(XtCallbackProc)prev_CB, (XtPointer)NULL);

  // Next button.
  xs = XmStringCreateSimple("Next");
  next_button = XtVaCreateWidget("nextbutton", xmPushButtonWidgetClass, form,
				 XmNlabelString, xs,
				 XmNleftAttachment, XmATTACH_WIDGET,
				 XmNleftWidget, prev_button,
				 NULL);
  XmStringFree(xs);
  XtAddCallback(next_button, XmNactivateCallback,
		(XtCallbackProc)next_CB, (XtPointer)NULL);
  
  // Stop button.
  xs = XmStringCreateSimple("Stop");
  stop_button = XtVaCreateWidget("stopbutton", xmPushButtonWidgetClass, form,
				 XmNlabelString, xs,
				 XmNleftAttachment, XmATTACH_WIDGET,
				 XmNleftWidget, next_button,
				 NULL);
  XmStringFree(xs);
  XtAddCallback(stop_button, XmNactivateCallback,
		(XtCallbackProc)stop_CB, (XtPointer)NULL);
  
  // Shuffle button.
  xs = XmStringCreateSimple("Shuffle");
  shuffle_button = XtVaCreateWidget("shufflebutton", xmPushButtonWidgetClass, form,
				 XmNlabelString, xs,
				 XmNleftAttachment, XmATTACH_WIDGET,
				 XmNleftWidget, stop_button,
				 NULL);
  XmStringFree(xs);
  XtAddCallback(shuffle_button, XmNactivateCallback,
		(XtCallbackProc)shuffle_CB, (XtPointer)NULL);
  
  // Sort button.
  xs = XmStringCreateSimple("Sort");
  sort_button = XtVaCreateWidget("sortbutton", xmPushButtonWidgetClass, form,
				 XmNlabelString, xs,
				 XmNleftAttachment, XmATTACH_WIDGET,
				 XmNleftWidget, shuffle_button,
				 NULL);
  XmStringFree(xs);
  XtAddCallback(sort_button, XmNactivateCallback,
		(XtCallbackProc)sort_CB, (XtPointer)NULL);

  // All button.
  xs = XmStringCreateSimple("All");
  all_button = XtVaCreateWidget("allbutton", xmPushButtonWidgetClass, form,
				 XmNlabelString, xs,
				 XmNleftAttachment, XmATTACH_WIDGET,
				 XmNleftWidget, sort_button,
				 NULL);
  XmStringFree(xs);
  XtAddCallback(all_button, XmNactivateCallback,
		(XtCallbackProc)all_CB, (XtPointer)NULL);
  
  // Textfield.
  search_text = XtVaCreateWidget("searchtext", xmTextWidgetClass, form,
				 XmNtopAttachment, XmATTACH_WIDGET,
				 XmNtopWidget, prev_button,
				 XmNleftAttachment, XmATTACH_FORM,
				 XmNrightAttachment, XmATTACH_FORM,
				 NULL);
  XtAddCallback(search_text, XmNactivateCallback,
		(XtCallbackProc)search_CB, (XtPointer)NULL);

  // Listview.
  Arg args[5];
  XtSetArg(args[0], XmNtopAttachment, XmATTACH_WIDGET);
  XtSetArg(args[1], XmNtopWidget, search_text);
  XtSetArg(args[2], XmNleftAttachment, XmATTACH_FORM);
  XtSetArg(args[3], XmNbottomAttachment, XmATTACH_FORM);
  XtSetArg(args[4], XmNrightAttachment, XmATTACH_FORM);
  listview = XmCreateScrolledList(form, "listview", args, 5);
  XtAddCallback(listview, XmNdefaultActionCallback,
		(XtCallbackProc)listpick_CB, (XtPointer)NULL);

  // Manage the widgets.
  XtManageChild(prev_button);
  XtManageChild(next_button);
  XtManageChild(stop_button);
  XtManageChild(shuffle_button);
  XtManageChild(sort_button);
  XtManageChild(search_text);
  XtManageChild(all_button);
  XtManageChild(listview);
  XtManageChild(form);

  // And show the panel.
  XtRealizeWidget(app_shell);
}

void interface_motif::main() {

  // FIXME
  pl->set_continuous(true);

  // Fill the listview.
  fill_list();

  // Enter the Motif event loop.
  XtAppMainLoop(app);

  // Stop playing.
  pl->stop();
}

void interface_motif::song_changed(int index) {

  // As this function is often called from a signal handler, you can't
  // use Motif functions directly, so instead we set up a timeout
  // which runs the next time Motif hits its event loop.
  XtAppAddTimeOut(app, 0, changesong_CB, (XtPointer)index);
}

